-- A set of different-looking buff bars. Inspired by Elkano's Buff Bars.
-- The player can left-click on a buff to cancel it.

-- Set our default placement values. If a SavedVariable file exists, then these will be overwritten after all our code is loaded. If it doesn't, then these will work as defaults.
placement = {x = 0, y = 0}
barHeight = 80
barWidth = 80
barAlpha = .5
role = 1
curTarget = nil
local debugmode = false
powerBuffs = {
	[1]={},
	[2]={},
	[3]={},
	[4]={},
	[5]={}
}
barPosx = {
	{650,650,925,925,550,1025},
	{650,650,925,925,550,1025},
	{650,650,925,925,550,1025},
	{650,650,925,925,550,1025},
	{650,650,925,925,550,1025}
}
barPosy = {
	{750,650,750,650,550,550},
	{750,650,750,650,550,550},
	{750,650,750,650,550,550},
	{750,650,750,650,550,550},
	{750,650,750,650,550,550}
}
barindex = 0
rightdown = false
--table that stores each bars font,alpha,color information for each role
barStats = {
	{
		{},
		{},
		{},
		{},
		{},
		{},
	},
	{
		{},
		{},
		{},
		{},
		{},
		{},
	},
	{
		{},
		{},
		{},
		{},
		{},
		{},
	},
	{
		{},
		{},
		{},
		{},
		{},
		{},
	},
	{
		{},
		{},
		{},
		{},
		{},
		{},
	},
}

-- global varable for moving buffs with the mouse and the table id
moveBuffs = false
moveBuffsTable = {function () resynch = true end, "PowerBuffs", "refresh"}

commandlist = "powerbuffs,powerMoveBuffs,powerrole,powermb,powerbuffinsert,powerstatinsert,powerstat"

-- Create our context so we can do rendering.
local context = UI.CreateContext("HUD")

-- Because it's currently impossible to destroy frames, we keep a reservoir of buff bars around. Buff bars work as a single unified whole, so once they're created, we can just call their "set" function.
local activeBuffBars = {}
local zombieBuffBars = {}

local lastAttach = nil  -- Attachment point for our next buff bar.
local function StartBuffBars()  -- Hide everything we have and start adding bars one at a time.
	for i , v in ipairs(activeBuffBars) do
  		table.insert(zombieBuffBars,i, v)
    	--v:SetVisible(false)
    	--v:SetPoint("TOPCENTER", UIParent, "TOPCENTER")
  	end

	activeBuffBars = {}
	lastAttach = nil
end

local function AddBuffBar(buff, time)
	local bar
	if zombieBuffBars[buff.index] then
		 bar = zombieBuffBars[buff.index]
		 zombieBuffBars[buff.index] = nil

	end

	if not bar then
	    -- We don't have any bars remaining, so we create a new one.
	    -- Our Bars are considered single objects that can be dealt with atomically. Each one has the functionality needed to update itself.
	    bar = UI.CreateFrame("Frame", "Bar", context)
	    -- Set an initial vertical pin to make some of our calculations work properly

	    bar:SetPoint("TOP", UIParent, "TOP")

	    bar:SetMouseMasking("limited")
	    bar.index = buff.index
		bar.isCooldown = buff.isCooldown

	    bar.text = UI.CreateFrame("Text", "Text", bar)
	    bar.stacks = UI.CreateFrame("Text", "Text", bar)
	    bar.timer = UI.CreateFrame("Text", "Timer", bar)
	    bar.icon = UI.CreateFrame("Texture", "Icon", bar)
	    -- Solid background - this is the actual "bar" part of it.
	    bar.solid = UI.CreateFrame("Frame", "Solid", bar)
	    bar.solid:SetLayer(-2)  -- Put it behind every other element.

	    --bar.text:SetText("test")
	    --bar.text:SetHeight(bar.text:GetFullHeight())  -- Set an initial height. We'll be overriding this, we just want to make sure it works.
	    bar.stacks:SetText("1")
	    bar.stacks:SetHeight(bar.stacks:GetFullHeight())  -- Set an initial height. We'll be overriding this, we just want to make sure it works.
	    bar.tar = buff.tar
		bar.id = buff.id



	    bar.stacks:SetPoint("TOPLEFT", bar.icon, "TOPLEFT")
	    bar.text:SetPoint("TOPLEFT", bar.icon, "TOPLEFT") -- The text is pinned to the top-right corner of the icon.

	    bar.timer:SetPoint("TOPRIGHT", bar.icon, "TOPRIGHT") -- The timer is pinned to the top-right corner.

	    bar.text:SetPoint("RIGHT", bar.timer, "LEFT") -- Make sure the text doesn't overrun the timer. We'll be changing the text's height based on the contents, but we'll leave the width calculated this way.



		-- The function that actually sets everything.
		function bar:SetBuff(buff, time)
		    --self:SetBackgroundColor(0.2, 0.2, 0.6, 0.3)
		    if barStats[role][buff.index]["barColor"] then
		    	self.solid:SetBackgroundColor(barStats[role][buff.index]["barColor"][1], barStats[role][buff.index]["barColor"][2], barStats[role][buff.index]["barColor"][3])
		    else
		    	self.solid:SetBackgroundColor(0.2, 0.2, 0.6)
		    end
			self.id = buff.id
			self.isCooldown = buff.isCooldown


		    --self.text:SetFontSize(60)
		    -- Display our stacking multiple.
			if buff.stack then
				self.stacks:SetText("" .. buff.stack)
				if barStats[role][buff.index]["stackFontSize"] then
		 			self.stacks:SetFontSize(barStats[role][buff.index]["stackFontSize"])
			 	else
			    	self.stacks:SetFontSize(32)
			    end

			    self.stacks:SetHeight(self.stacks:GetFullHeight())
		   		self.stacks:SetWidth(self.stacks:GetFullWidth())

				self.stacks:SetVisible(true)
			else
				self.stacks:SetVisible(false)
			end

		    if barStats[role][buff.index]["lBarWidth"] then
		 		bar:SetWidth(barStats[role][buff.index]["lBarWidth"])
		 	else
		    	bar:SetWidth(barWidth)
		    end
		    if barStats[role][buff.index]["lBarHeight"] then
		 		bar:SetHeight(barStats[role][buff.index]["lBarHeight"])
		 	else
		    	bar:SetHeight(barHeight)
		    end

		    if barStats[role][buff.index]["timerFontSize"] then
		 		self.timer:SetFontSize(barStats[role][buff.index]["timerFontSize"])
		 	else
		    	self.timer:SetFontSize(18)
		    end

		    self.timer:SetHeight(self.timer:GetFullHeight())

		    if barStats[role][buff.index]["stacksFontColor"] then
		 		self.stacks:SetFontColor(barStats[role][buff.index]["stacksFontColor"][1],barStats[role][buff.index]["stacksFontColor"][2],barStats[role][buff.index]["stacksFontColor"][3])
		 	else
		    	self.stacks:SetFontColor(1,.07,.58)
		    end
		    if barStats[role][buff.index]["timerFontColor"] then
		 		self.timer:SetFontColor(barStats[role][buff.index]["timerFontColor"][1],barStats[role][buff.index]["timerFontColor"][2],barStats[role][buff.index]["timerFontColor"][3])
		 	else
		    	self.timer:SetFontColor(0,0,0)
		    end


		  	if barStats[role][buff.index]["iconAlpha"] then
		  		self.icon:SetAlpha(barStats[role][buff.index]["iconAlpha"])
		  	else
		  		self.icon:SetAlpha(barAlpha)
		  	end
		  	if barStats[role][buff.index]["stacksAlpha"] then
		  		self.stacks:SetAlpha(barStats[role][buff.index]["stacksAlpha"])
		  	else
		  		self.stacks:SetAlpha(1)
		  	end
		  	if barStats[role][buff.index]["timerAlpha"] then
		  		self.timer:SetAlpha(barStats[role][buff.index]["timerAlpha"])
		  	else
		  		self.timer:SetAlpha(1)
		  	end

			if barStats[role][buff.index]["trackAbsence"] == true then
			 	--buffs are being tracked for absence
			 	if not buff.id then
			 		--buff isn't on player
						self.completion = nil

						-- This is a permanent buff without a timer, so don't show any of that.
						self.solid:SetVisible(false)
						self.timer:SetVisible(false)
						self.text:SetVisible(false)
						self.icon:SetVisible(true)
						self.timer:SetWidth(0)
			 	else
			 		--buff is on player so hide everything
			 		self.solid:SetVisible(false)
					self.timer:SetVisible(false)
					self.text:SetVisible(false)
					self.icon:SetVisible(false)
					self.stacks:SetVisible(false)
					self.timer:SetWidth(0)
			 	end

			else
				 --buff is not being tracked for absence
				if not buff.id then
			 		--buff isn't on player so hide everything
			 		self.solid:SetVisible(false)
					self.timer:SetVisible(false)
					self.text:SetVisible(false)
					self.icon:SetVisible(false)
					self.stacks:SetVisible(false)
					self.timer:SetWidth(0)
			 	else
			 		--buff is on player
			 		if buff.duration then
					self.completion = buff.begin + buff.duration
					self.duration = buff.duration
					local	remaining = self.completion - time


					-- Display everything we might have hidden.
					self.solid:SetVisible(true)
					self.timer:SetVisible(true)
					self.text:SetVisible(false)
					self.icon:SetVisible(true)
					self:Tick(time)
					else
						self.completion = nil

						-- This is a permanent buff without a timer, so don't show any of that.
						self.solid:SetVisible(false)
						self.timer:SetVisible(false)
						self.text:SetVisible(false)
						self.icon:SetVisible(true)
						self.timer:SetWidth(0)

					end

			 	end
			end


		  if barStats[role][buff.index]["vBar"] then
		  		self.icon:ClearAll()
		    	self.icon:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT") -- The icon is pinned to the top-left corner of the bar.
		    	self.icon:SetPoint("BOTTOMRIGHT", self,"BOTTOMRIGHT")
		    	self.icon:SetPoint("TOPLEFT",self,0,1-(self.icon:GetWidth() / bar:GetHeight()))
		    	self.icon:SetPoint("TOPRIGHT",self,1,1-(self.icon:GetWidth() / bar:GetHeight()))
			    self.icon:SetLayer(-1)
			    if self.completion then
				    remaining = self.completion - time
				    self.solid:SetPoint("BOTTOMLEFT",self,0,1-self:GetWidth()/self:GetHeight())
			      	self.solid:SetPoint("BOTTOMRIGHT",self,1,1-self:GetWidth()/self:GetHeight())
			      	self.solid:SetPoint("TOPLEFT", self, 0 , ((1-remaining / self.duration) * (1-self:GetWidth()/self:GetHeight())))
			      	self.solid:SetPoint("TOPRIGHT", self, 1 ,((1-remaining / self.duration) * (1-self:GetWidth()/self:GetHeight())))
			    end
			    self.solid:SetAlpha(barAlpha)
			    --self.solid:SetVisible(false)

			    -- Re-square and set the icon
				if barStats[role][buff.index]["icon"] then
					self.icon:SetTexture("Rift", barStats[role][buff.index]["icon"])
			  	else
			  		if buff.icon then self.icon:SetTexture("Rift", buff.icon)end
			  	end
				self.icon:SetHeight(self.icon:GetWidth())
		    elseif barStats[role][buff.index]["hBar"] then
		    	self.icon:SetPoint("TOPLEFT", self, "TOPLEFT") -- The icon is pinned to the top-left corner of the bar.
			    self.icon:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT") -- Vertically, it always fills the entire bar.
			    self.icon:SetPoint("TOPRIGHT",self,(self.icon:GetHeight() / bar:GetWidth()),0)
		    	self.icon:SetPoint("BOTTOMRIGHT",self,(self.icon:GetHeight() / bar:GetWidth()),1)
			    self.icon:SetLayer(-1)
			    if self.completion then
			    	remaining = self.completion - time
				    self.solid:SetPoint("BOTTOMLEFT",self,self:GetWidth()/self:GetHeight(),0)
			      	self.solid:SetPoint("TOPLEFT",self,self:GetWidth()/self:GetHeight(),0)
			      	self.solid:SetPoint("RIGHT", self, (remaining / self.duration * (1-self:GetHeight()/self:GetWidth())) + self:GetHeight()/self:GetWidth(), nil)
			    end
			    self.solid:SetAlpha(barAlpha)
			    -- Re-square and set the icon
				if barStats[role][buff.index]["icon"] then
					self.icon:SetTexture("Rift", barStats[role][buff.index]["icon"])
			  	else
			  		if buff.icon then self.icon:SetTexture("Rift", buff.icon)end
			  	end
			  	self.icon:SetWidth(self.icon:GetHeight())
		    else
		    	self.icon:SetPoint("TOPLEFT", self, "TOPLEFT") -- The icon is pinned to the top-left corner of the bar.
			    self.icon:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT") -- Vertically, it always fills the entire bar.
			    self.icon:SetLayer(-1)
			    self.solid:SetAlpha(0)

			    -- Re-square and set the icon
				if barStats[role][buff.index]["icon"] then
					self.icon:SetTexture("Rift", barStats[role][buff.index]["icon"])
			  	else
			  		if buff.icon then self.icon:SetTexture("Rift", buff.icon)end
			  	end
				self.icon:SetWidth(self.icon:GetHeight())
		    end


			self.debuff = buff.debuff
			self.buffid = buff.id
		end

		-- This is our update function, called once every frame.
		function bar:Tick(time)
			if self.isCooldown then
			    --self:SetVisible(false)
			end
			if self.completion and self.id then
		  		local	remaining = self.completion - time
			    if remaining < 0 then
			        self.timer:SetVisible(false)
			        self.solid:SetVisible(false)
				    if self.isCooldown then
				        self:SetVisible(false)
						self.icon:SetVisible(false)
						self.stacks:SetVisible(false)
				    end

			    else
			      -- Update our timer.

			      if barStats[role][buff.index]["vBar"]  then
			      	--print(remaining / self.duration)
			      	self.solid:ClearAll()
			      	self.solid:SetPoint("BOTTOMLEFT",self,0,1-self:GetWidth()/self:GetHeight())
			      	self.solid:SetPoint("BOTTOMRIGHT",self,1,1-self:GetWidth()/self:GetHeight())
			      	self.solid:SetPoint("TOPLEFT", self, 0 , ((1-remaining / self.duration) * (1-self:GetWidth()/self:GetHeight())))
			      	self.solid:SetPoint("TOPRIGHT", self, 1 ,((1-remaining / self.duration) * (1-self:GetWidth()/self:GetHeight())))
			      	self.solid:SetVisible(true)
			      elseif barStats[role][buff.index]["hBar"] then
			      	--self.solid:ClearAll()
			      	self.solid:SetPoint("BOTTOMLEFT",self,self:GetHeight()/self:GetWidth(),1)
			      	self.solid:SetPoint("TOPLEFT",self,self:GetHeight()/self:GetWidth(),0)
			      	--self.solid:SetPoint("RIGHT",self,"RIGHT")
			      	self.solid:SetPoint("RIGHT", self, ((remaining / self.duration) * (1-self:GetHeight()/self:GetWidth())) + self:GetHeight()/self:GetWidth(), nil)
			      	self.solid:SetVisible(true)
			      else
			      	--self.solid:SetVisible(false)

			      end

			      -- Generate the timer text string.
			      if remaining >= 3600 then
			        self.timer:SetText(string.format("%d:%02d:%02d", math.floor(remaining / 3600), math.floor(remaining / 60) % 60, math.floor(remaining) % 60))
			      else
			        self.timer:SetText(string.format("%d:%02d", math.floor(remaining / 60), math.floor(remaining) % 60))
			      end
			      self.timer:SetWidth(self.timer:GetFullWidth())
				end
			else
				--self:SetVisible(false)
			end


			if moveBuffs == true then
				function bar.Event:RightDown()
				    if rightdown == false then
				    	--print(self.index)
				    	barindex = self.index
				    	rightdown = true
				    end
				end
			    function bar.Event:RightUp()
			    	rightdown = false
			    end
			    function bar.Event:RightUpoutside()
			    	rightdown = false
			    end
			    function bar.Event:MouseMove(x,y)
			    	if rightdown and moveBuffs then
			    		barPosx[role][barindex] = math.floor(x)
			    		barPosy[role][barindex] = math.floor(y)
			    		fullRefresh = true
			    	end
			    end
			end

  		end
    end
	-- Show the bar and set the data.
	bar:SetVisible(true)
	bar:SetBuff(buff, time)


    if barStats[role][buff.index]["vBar"] then
    	bar:SetPoint("TOPLEFT", UIParent, "TOPLEFT", barPosx[role][buff.index], barPosy[role][buff.index])
	elseif barPosx[role][buff.index] then
		bar:SetPoint("TOPLEFT", UIParent, "TOPLEFT", barPosx[role][buff.index], barPosy[role][buff.index])
	else
		bar:SetPoint("TOPLEFT", UIParent, "TOPLEFT", placement.x, placement.y)
		table.insert(barPosx[role],buff.index,0)
		table.insert(barPosy[role],buff.index,0)
	end
	table.insert(activeBuffBars,buff.index, bar)
end

local function refresh(time)
    -- Get the list of buffs from the player. Note that "player" can be replaced by "player.target" to turn this into a target's-buff-bar display.
    local bufflist = Inspect.Buff.List("player")
    local tarBuffList
    local abilityList = Inspect.Ability.List()
    if curTarget then tarBuffList = Inspect.Buff.List(curTarget) end

    if not bufflist then return end -- If we don't get anything, then we don't currently have information about the player. This may happen when the player is logging in or teleporting long distances.

    --process target buffs if and only if we have a valid target :)

    local buffdetails = Inspect.Buff.Detail("player", bufflist) -- Retrieve the details for all of our buffs. If there were only certain buffs we cared about, we could filter them out before asking for detail.
    local tarBuffDetails = {}

    if curTarget and tarBuffList then tarBuffDetails = Inspect.Buff.Detail(curTarget,tarBuffList) end

    StartBuffBars() -- Reset the buff bars and start entering buffs.

    -- buff id may be needed if canceling buffs is disired
    local bbars = {}
    local tarbbars = {}
    local abars = {}

    for k= 1, table.maxn(powerBuffs[role]) do
    	inbbars = false
    	intarbbars = false
		local inabars = false

	    if powerBuffs[role][k] then
	    	local p = powerBuffs[role][k]



	    	if tarBuffDetails and curTarget and barStats[role][k]["isTarget"] then
	    	    for id,v in pairs(tarBuffDetails) do
		  	        if v then
		  	        	if not (Inspect.Unit.Lookup("player") == v.caster) and v.debuff then  --if its not my debuff
		  	        		if not v.debuff  then  -- and if its not his buff then remove it
			          			--print(Inspect.Buff.Detail(curTarget,v).name)
			          			--print(curTarget)
			          			--print(Inspect.Buff.Detail(curTarget,v).caster)
			          			tarBuffDetails[id] = nil
			          		end
		  	        	end
		  	        end
		  	        --add target buff icons to memory to allow tracking absence

			        if v["name"] == p  then
			            inlist = true
			            barStats[role][k]["icon"] = v["icon"]
  			        end

			    end

	    	end




	    end
    end

    for id, bufff in pairs(buffdetails) do
        bufff.id = id  -- Make a copy of the ID, because we'll need it
        bbars[bufff.name]=bufff
    end
    for id, bufff in pairs(tarBuffDetails) do
        bufff.id = id  -- Make a copy of the ID, because we'll need it
        tarbbars[bufff.name]= bufff
    end
    for k = 1,table.maxn(powerBuffs[role]) do
        local p = powerBuffs[role][k]

        if not bbars[p] then
	    	--create the buff and add it to bbars
	    	local buff = {}
	    	if barStats[role][k]["icon"] then buff["icon"] = barStats[role][k]["icon"]end
	    	buff["name"] = p
	    	buff.completion = nil

	    	bbars[p]=buff
        end

        if not tarbbars[p] and barStats[role][k]["isTarget"] then
	    	--create the buff and add it to tarbbars
	    	local buff = {}
	    	if barStats[role][k]["icon"] then buff["icon"] = barStats[role][k]["icon"]end
	    	buff["name"] = p
	    	buff["isTarget"] = true
	    	buff.completion = nil
	    	tarbbars[p]=buff
	    end


        if barStats[role][k]["aId"] and barStats[role][k]["cooldown"] then
            local buff =  Inspect.Ability.Detail(barStats[role][k]["aId"])
            if buff then
                buff.id = barStats[role][k]["aId"]
                abars[buff.name]=buff
            end


        end
    end



  	-- Now that we have the ordering, we just add the bars one at a time. Done!
  	for cunt = 1,table.maxn(powerBuffs[role]) do
	  	index = 0
		local v = {}
	  	if powerBuffs[role][cunt] then
	  	    if not barStats[role][cunt]["isTarget"] and not barStats[role][cunt]["cooldown"] then
	  	    	v = bbars[powerBuffs[role][cunt]]
	  	    	v.index = cunt
	  	    	v.tar = "player"
	  	    	AddBuffBar(v, time)
	  	    end
		end


	  	if powerBuffs[role][cunt] and barStats[role][cunt]["cooldown"] then
			local v = abars[powerBuffs[role][cunt]]
	  		if v then
	  		    v.index = cunt
	  		    v.tar = "player"
			    v.begin = v.currentCooldownBegin
			    v.duration = v.currentCooldownDuration
			    if not v.begin then
			        v.id = nil
			    end
	  		    v.isCooldown = true
	  		    AddBuffBar(v, time)
			end
	  	end
  	end
	if curTarget then
		--add target buffs/debuffs
		for cunt = 1,table.maxn(powerBuffs[role]) do

		  	if powerBuffs[role][cunt] then
		  		if tarbbars[powerBuffs[role][cunt]]  and barStats[role][cunt]["isTarget"] then
		  			local v = tarbbars[powerBuffs[role][cunt]]
		  			v.index = cunt
		  			v.tar = curTarget
		  			AddBuffBar(v, time)
		  		end
		  	end
	  	end
	end
end

local resynch = false
local function tick()
  if debugmode then
       for k,v in pairs(Inspect.Addon.Cpu()) do
		    if k == "PowerBuffs" then
		        local sum = 0
		    	for k,v in pairs(v) do
		    	    sum = sum + v
		    		if k == "refresh" then
		    		end
		    	end
		    	if sum then
		    	    print(sum)
		    	end

		    end
		end
  end

  if resynch or moveBuffs then
    -- A recalculation has been queued, so go ahead and recalculate.
    refresh(Inspect.Time.Frame())
    resynch = false
  else
    -- Just do a tick refresh
    local time = Inspect.Time.Frame()

	for _, v in ipairs(activeBuffBars) do
	  v:Tick(time)
	end

  end
end

local function checkBuffs(unit,buffs)
    local time = Inspect.Time.Frame()
    bDetails = {}
    for i in pairs(buffs) do
        v = Inspect.Buff.Detail(unit,i)
        v.id = i
        bDetails[v.name] = v
    end

    if unit == Inspect.Unit.Lookup("player") then
        for j,k in ipairs(powerBuffs[role]) do
                if bDetails[k] then
                    resynch = true
                    return
                end

        end

    else
        for j,k in ipairs(powerBuffs[role]) do

            if not barStats[role][j]["isTarget"] then
                if (unit == curTarget) then
                    if bDetails[k] then
                        resynch = true
                        return
                    end
                end
            end

        end

    end

 end

table.insert(Event.Buff.Add, {checkBuffs, "PowerBuffs", "refresh"})
table.insert(Event.Buff.Change, {checkBuffs, "PowerBuffs", "refresh"})
table.insert(Event.Buff.Remove, {
 function (unit,buffs)
    if  unit == Inspect.Unit.Lookup("player") then
        resynch = true
    end
 end
, "PowerBuffs", "refresh"})

table.insert(Event.Ability.Cooldown.Begin,{function (cooldowns)
    local abars = {}
    for i,c in pairs(cooldowns) do
        local v = Inspect.Ability.Detail(i)
        if v then
            v.id = i
            abars[v.name] = v
        end
    end

    for k = 1,table.maxn(powerBuffs[role]) do
        p = powerBuffs[role][k]

        if barStats[role][k] then
            if abars[p] then
                if barStats[role][k]["cooldown"] then
                    barStats[role][k]["aId"] = abars[p].id
                end

            end
        end

    end
    resynch = true
 end, "PowerBuffs", "refresh"})

table.insert(Event.Ability.Target, {function (abilities)
		for k,v in pairs(abilities) do
			curTarget = v
			break
		end
		resynch = true
end, "PowerBuffs", "target change"})

-- Our update event. This triggers every single frame.
table.insert(Event.System.Update.Begin, {tick, "PowerBuffs", "refresh"})


-- Our buffs management command.  buffs are added to the currently set role(defaults to 1)
-- Parameters: comma deliminated string of buff names.(extra spaces may cause problems)
local function buffs(commandline)
	if commandline == "" then
		print("adds a list of buffs to the current role.  Use /powerrole to change the current role\n")
		print("powerbuffs [option] [buffs]...")
		print("options:\nhelp\n    Prints a list of commands\nrefresh\n    readds all buffs in the current role")
		print("buffs: comma diliminated list of buff names\n    Example: /powerbuffs Salvation,Reparation,Lurking Decay,Decay")
		print("     Note: changes to buffs will be rendered the next time a buff change event is triggered")
	elseif commandline == "help" then
		print("Commands:\n" .. commandlist .. "\nType each command without parameters to get detailed info")
	elseif commandline == "print" then
		print("Role: " .. role)
		if not powerBuffs[role] then table.insert(powerBuffs,role,{}) end
		if table.maxn(powerBuffs[role]) > 0 then
			for i = 1, table.maxn(powerBuffs[role]) do
				if powerBuffs[role][i] then
					print("index: " .. i .. " Buff Name: " .. powerBuffs[role][i])
				else
					print("index: " .. i .. " Buff Name: ...")
				end
			end
		end
	elseif commandline == "refresh" then
		if not powerBuffs[role] then table.insert(powerBuffs,role,{}) end
		buffys = {}
		for i = 1, table.maxn(powerBuffs[role]) do
			table.insert(buffys,i,powerBuffs[role][i])
			if not barStats[role][i] then
				table.insert(barStats[role],i,{})
			end
		end
		powerBuffs[role] = buffys

	else
		buffys = {}
		for token in string.gmatch(commandline, "[^,]+") do
			table.insert(buffys,token)

		end
		if powerBuffs[role] then
			table.remove(powerBuffs,role)
		end
		table.insert(powerBuffs,role,buffys)
		for i = 1,table.maxn(powerBuffs[role]) do
			if not barStats[role][i] then
				table.insert(barStats[role],i,{})
			end
		end
	end
end

local function move(commandline)
	if commandline == "off" then
		moveBuffs = false
	elseif commandline == "" then
		print("powermovebuffs status\nStatus: on or off")
		print("Currently, this is buggy due to mouse events and has been disabled. /powermb is recommended")
	elseif commandline == "on" then
		moveBuffs = true
		print(moveBuffs)
	end
end

-- Our role set function
-- Parameters: int role
--				1-n representing your current role
local function powerrole(commandline)
	if commandline == "" then
		print("sets the current configuration role")
		print("/powerrole roleId")
		print("roleId:\n    1-n integer")
		print("Example: /powerRole 2")
		print("\nCurrent Role: ".. role)
	else
		role = tonumber(commandline:match("(%d+)"))
		if not powerBuffs[role] then table.insert(powerBuffs,role,{}) end
		if not barStats[role] then table.insert(barStats,role,{}) end
		if not barPosx[role] then table.insert(barPosx,role,{}) end
		if not barPosy[role] then table.insert(barPosy,role,{}) end
		print(role)
	end
end

local function powermb(commandline)
	if commandline == "" then
		print("powermb roleid, xDiff, yDiff,[barIds]...")
		print("roleid:\n    1-n representing the role to change where n is the current number of roles or all")
		print("xDiff:\n    integer value represting the relative distance to move the buff in pixels")
		print("yDiff:\n    integer value represting the relative distance to move the buff in pixels")
		print("barIds:\n    comma diliminated list of integers 1-n where n is the current number of bars")
		print("")
		print("Example: /powermb all, 100, 0, 1, 2, 5")
		print("moves left side buffs along x by 100 and y by 0 pixels for all roles")
	elseif commandline:match("print") then
		print(barPosx[role][tonumber(commandline:match("%d+"))])
		print(barPosy[role][tonumber(commandline:match("%d+"))])
	else
		tokens = {}
		for token in string.gmatch(commandline, "[^,]+") do
			table.insert(tokens,token)
		end
		xdiff = tonumber(tokens[2]:match("-*%d+"))
		ydiff = tonumber(tokens[3]:match("-*%d+"))
		if tokens[1] == "all" then
			for j = 1, table.maxn(barPosx) do
				for i = 4, table.maxn(tokens) do
					if not  barPosx[j][tonumber(tokens[i])]then
						barPosx[j][tonumber(tokens[i])] = 0
					end
					if not  barPosy[j][tonumber(tokens[i])]then
						barPosy[j][tonumber(tokens[i])] = 0
					end
					barPosx[j][tonumber(tokens[i])] = barPosx[j][tonumber(tokens[i])] + xdiff
					barPosy[j][tonumber(tokens[i])] = barPosy[j][tonumber(tokens[i])] + ydiff
				end
			end
		else
			roletemp = tonumber(tokens[1]:match("(%d+)"))
			for i = 4, table.maxn(tokens) do
				if not  barPosx[roletemp][tonumber(tokens[i])]then
					barPosx[roletemp][tonumber(tokens[i])] = 0
				end
				if not  barPosy[roletemp][tonumber(tokens[i])]then
					barPosy[roletemp][tonumber(tokens[i])] = 0
				end
				barPosx[roletemp][tonumber(tokens[i])] = barPosx[roletemp][tonumber(tokens[i])] + xdiff
				barPosy[roletemp][tonumber(tokens[i])] = barPosy[roletemp][tonumber(tokens[i])] + ydiff
			end
		end
	end
end

local function powerbuffinsert(commandline)
	if commandline == "" then
		print("inserts a single buff to a specified bar.  \nUsefull when you want to change a single buff.  Use /powerBuffs to add the whole list\n")
		print("powerbuffinsert barId,BuffName")
		print("barId:\n    integer 1-n representing the bar to bind the buff to")
		print("BuffName:\n     string Name of buff (no leading / trailing spaces)\n")
		print("Example: /powerbuffinsert 5,Conviction")
	else
		tokens = {}
		for token in string.gmatch(commandline, "[^,]+") do
			table.insert(tokens,token)
		end
		local id = tonumber(tokens[1]:match("%d+"))
		if not barStats[role][id] then
			table.insert(barStats[role],id,{})
		end
		table.remove(powerBuffs[role],id)
		table.insert(powerBuffs[role],id,tokens[2])
	end
end

local function powerStatInsert(commandline)
	if commandline == "" then
		print("powerstatinsert [options] barId,IconAlpha,StackAlpha,TimerAlpha,StackFontColor,TimerFontColor,StackFontSize,TimerFontSize,BarWidth,BarHeight,TrackAbsence,TrackOnTarget,BarVerticle,BarHorizontal")
		print("option:\nprint\n    prints the values for the specified bar")
		print("Example: /powerstatinsert print,6")
		print("")
		print("barId:    *mandatory* integer 1-n representing the bar to bind the stats to")
		print("IconAlpha:   0-1 inclusive 0 being fully transparent")
		print("StackAlpha:   0-1 inclusive 0 being fully transparent")
		print("TimerAlpha:   0-1 inclusive 0 being fully transparent")
		print("StackFontColor:   red green blue 0-1")
		print("TimerFontColor:   red green blue 0-1")
		print("StackFontSize:   1-n integer")
		print("TimerFontSize:   1-n integer")
		print("BarWidth: 1-n pixels")
		print("BarHeight: 1-n pixels")
		print("TrackAbsence:  yes or no   yes specifies that the buff will be rendered if it is not on the player.  Defaults to no.")
		print("TrackOnTarget: yes or no    yes specifies that the buff will tracked on the current target.  Defaults to no")
		print("BarVerticle:  yes or no   yes specifies that the bar will verticle. Defaults to no")
		print("BarHorizontal: yes or no   yes specifies that the bar will be horizontal. Defaults to no")
		print("Example: /powerstatinsert 5,.5,1,1,1 .07 .58,0 0 0,32,18,80,80,no")
		print("comma diliminated, if changing an optional field is not disired simply put a space in that field")
		print("Example: /powerstatinsert 5,.5,1,1, , , , ,80,80,no")

	else
		tokens = {}
		for token in string.gmatch(commandline, "[^,]+") do
			table.insert(tokens,token)
		end
		rgb = {}
		barId = tonumber(tokens[1]:match("%d+"))

		if commandline:match("print") then
			for k,v in pairs(barStats[role][tonumber(tokens[2]:match("%d+"))]) do
				if type(v) == "table" then
					print(k .. ": " .. table.concat(v,","))
				elseif type(v) == "boolean" then
					if v then
						print(k .. ": yes")
					else
						print(k .. ": no")
					end
				else
					print(k .. ": " .. v)
				end
			end
		elseif commandline:match("IconAlpha") then
			if tokens[2] and tokens[2]:match(".*%d+") then barStats[role][barId]["iconAlpha"] = tonumber(tokens[2]:match(".*%d+"))end
		elseif commandline:match("StackAlpha") then
			if tokens[2] and tokens[2]:match(".*%d+") then barStats[role][barId]["stackAlpha"] = tonumber(tokens[2]:match(".*%d+"))end
		elseif commandline:match("TimerALpha") then
			if tokens[2] and tokens[2]:match(".*%d+") then barStats[role][barId]["timerAlpha"] = tonumber(tokens[2]:match(".*%d+"))end
		elseif commandline:match("StackFontColor") then
			rgb = {}
			if tokens[2] then
				for token in string.gmatch(tokens[2],"[^ ]+") do
					table.insert(rgb,tonumber(token))
				end
				if table.maxn(rgb) == 3 then barStats[role][barId]["stackFontColor"] = rgb end
			end
		elseif commandline:match("TimerFontColor") then
			rgb = {}
			if tokens[2] then
				for token in string.gmatch(tokens[2],"[^ ]+") do
					table.insert(rgb,tonumber(token))
				end
				if table.maxn(rgb) == 3 then barStats[role][barId]["timerFontColor"] = rgb end
			end
		elseif commandline:match("StackFontSize") then
			if tokens[2] and tokens[2]:match("%d+") then barStats[role][barId]["stackFontSize"] = tonumber(tokens[2]:match("%d+"))end
		elseif commandline:match("TimerFontSize") then
			if tokens[2] and tokens[2]:match("%d+") then barStats[role][barId]["timerFontSize"] = tonumber(tokens[2]:match("%d+"))end
		elseif commandline:match("BarWidth") then
			if tokens[2] and tokens[2]:match("%d+") then barStats[role][barId]["lBarWidth"] = tonumber(tokens[2]:match("%d+"))end
		elseif commandline:match("BarHeight") then
			if tokens[2] and tokens[2]:match("%d+") then barStats[role][barId]["lBarHeight"] = tonumber(tokens[2]:match("%d+"))end
		elseif commandline:match("TrackAbsence") then
			if tokens[2] and tokens[2]:match("yes") then
				barStats[role][barId]["trackAbsence"] = true
			elseif tokens[2] then
				barStats[role][barId]["trackAbsence"] = nil
			end
		elseif commandline:match("isTarget") then
			if tokens[2] and tokens[2]:match("yes") then
				barStats[role][barId]["isTarget"] = true
			elseif tokens[2] then
				barStats[role][barId]["isTarget"] = nil
			end
		elseif commandline:match("vBar") then
			if tokens[2] and tokens[2]:match("yes") then
				barStats[role][barId]["vBar"] = true
			elseif tokens[2] then
				barStats[role][barId]["vBar"] = nil
			end
		elseif commandline:match("hBar") then
			if tokens[2] and tokens[2]:match("yes") then
				barStats[role][barId]["hBar"] = true
			elseif tokens[2] then
				barStats[role][barId]["hBar"] = nil
			end

		elseif commandline:match("BarColor") then
			rgb = {}
			if tokens[2] then
				for token in string.gmatch(tokens[2],"[^ ]+") do
					table.insert(rgb,tonumber(token))
				end
				if table.maxn(rgb) == 3 then barStats[role][barId]["barColor"] = rgb end
			end
		else

			if tokens[2] and tokens[2]:match(".*%d+") then barStats[role][barId]["iconAlpha"] = tonumber(tokens[2]:match(".*%d+"))end
			if tokens[3] and tokens[3]:match(".*%d+") then barStats[role][barId]["stackAlpha"] = tonumber(tokens[3]:match(".*%d+"))end
			if tokens[4] and tokens[4]:match(".*%d+") then barStats[role][barId]["timerAlpha"] = tonumber(tokens[4]:match(".*%d+"))end
			if tokens[5] then
				for token in string.gmatch(tokens[5],"[^ ]+") do
					table.insert(rgb,tonumber(token))
				end

				if table.maxn(rgb) == 3 then barStats[role][barId]["stackFontColor"] = rgb end
			end
			rgb = {}
			if tokens[6] then
				for token in string.gmatch(tokens[6],"[^ ]+") do
					table.insert(rgb,tonumber(token))
				end
				if table.maxn(rgb) == 3 then barStats[role][barId]["timerFontColor"] = rgb end
			end
			if tokens[7] and tokens[7]:match("%d+") then barStats[role][barId]["stackFontSize"] = tonumber(tokens[7]:match("%d+")) end
			if tokens[8] and tokens[8]:match("%d+") then barStats[role][barId]["timerFontSize"] = tonumber(tokens[8]:match("%d+")) end
			if tokens[9] and tokens[9]:match("%d+") then barStats[role][barId]["lBarWidth"] = tonumber(tokens[9]:match("%d+")) end
			if tokens[10] and tokens[10]:match("%d+") then barStats[role][barId]["lBarHeight"] = tonumber(tokens[10]:match("%d+")) end
			if tokens[11] and tokens[11]:match("yes") then
				barStats[role][barId]["trackAbsence"] = true
			elseif tokens[11] then
				barStats[role][barId]["trackAbsence"] = nil
			end
			if tokens[12] and tokens[12]:match("yes") then
				barStats[role][barId]["isTarget"] = true
			elseif tokens[12] then
				barStats[role][barId]["isTarget"] = nil
			end
			if tokens[13] and tokens[13]:match("yes") then
				barStats[role][barId]["vBar"] = true
			elseif tokens[13] then
				barStats[role][barId]["vBar"] = nil
			end
			if tokens[14] and tokens[14]:match("yes") then
				barStats[role][barId]["hBar"] = true
			elseif tokens[14] then
				barStats[role][barId]["hBar"] = nil
			end
			rgb = {}
			if tokens[15] then
				for token in string.gmatch(tokens[15],"[^ ]+") do
					table.insert(rgb,tonumber(token))
				end
				if table.maxn(rgb) == 3 then barStats[role][barId]["barColor"] = rgb end
			end
		end
	end
end

local function powerStatGlobal(commandline)
	if commandline == "" then
		print("sets varables that can effect all of the bars")
		print("powerstart [Options] iconAlpha,barWidth,barHeight")
		print("Options:")
		print("    print:   prints the global values")
		print("IconAlpha:   0-1 inclusive 0 being fully transparent")
		print("BarWidth: 1-n pixels")
		print("BarHeight: 1-n pixels")
		print("Example: /powerstat .5,80,80")

	elseif commandline:match("print") == "print" then
		print("Bar Alpha: " .. barAlpha ..  "\nBar Width: " .. barWidth .. "\nBar Height: " .. barHeight)
	elseif commandline:match("debug") == "debug" then
	    if debugmode then
	        debugmode = false
	    else
	        debugmode = true
		end
		if powergui.debugmode then
		    powergui.debugmode = false
		else
		    powergui.debugmode = true
		end
		print(debugmode)
	else
		tokens = {}
		for token in string.gmatch(commandline, "[^,]+") do
			table.insert(tokens,token)
		end
		barAlpha = tonumber(tokens[1]:match(".*%d+"))
		barWidth = tonumber(tokens[2]:match("%d+"))
		barHeight = tonumber(tokens[3]:match("%d+"))
	end
end

local function cpu()
	for k,v in pairs(Inspect.Addon.Cpu()) do
		if k == "PowerBuffs" then
			for k,v in pairs(v) do
				print(k)
				print(v)
			end
		end
	end
end
table.insert(Command.Slash.Register("powerbuffscpu"), {cpu, "PowerBuffs", "buffs"})
table.insert(Command.Slash.Register("powerbuffs"), {buffs, "PowerBuffs", "buffs"})
table.insert(Command.Slash.Register("powerMoveBuffs"), {move, "PowerBuffs", "Move Buffs"})
table.insert(Command.Slash.Register("powerrole"), {powerrole, "PowerBuffs", "change roles"})
table.insert(Command.Slash.Register("powermb"), {powermb, "PowerBuffs", "moven those buffs"})
table.insert(Command.Slash.Register("powerbuffinsert"), {powerbuffinsert, "PowerBuffs", "moven those buffs"})
table.insert(Command.Slash.Register("powerstatinsert"), {powerStatInsert, "PowerBuffs", "configure those buffs"})
table.insert(Command.Slash.Register("powerstat"), {powerStatGlobal, "PowerBuffs", "configure those buffs"})
